var MICRO = "\xB5";

// eslint-disable-next-line @typescript-eslint/no-explicit-any
function isFunction(x) {
  return typeof x === 'function';
}

function isNumber(value) {
  return typeof value === 'number';
}

var isArray = Array.isArray;

var pow = Math.pow;

function transformFindUnitArray(units, base) {
  return units.map(function (item) {
    return {
      pre: item.pre,
      div: pow(base, item.exp)
    };
  });
}

function sortFindUnitArray(units, base) {
  return transformFindUnitArray(units, base).sort(function (a, b) {
    return b.div - a.div;
  });
}

var unity = {
  pre: '',
  div: 1
};
var defaultFindResults = [{
  exp: 4,
  pre: 'T'
}, {
  exp: 3,
  pre: 'G'
}, {
  exp: 2,
  pre: 'M'
}, {
  exp: 1,
  pre: 'K'
}, {
  exp: 0,
  pre: ''
}, {
  exp: -1,
  pre: 'm'
}, {
  exp: -2,
  pre: MICRO
}, {
  exp: -3,
  pre: 'n'
}, {
  exp: -4,
  pre: 'p'
}, {
  exp: -5,
  pre: 'f'
}];
function createUnitFinder(find, table) {
  if (isFunction(find)) {
    return function (value) {
      var result = find(value);

      if (typeof result !== 'object') {
        throw new TypeError(result + " is not a valid return value for \"find\" option");
      }

      return result;
    };
  }

  var results = find ? isNumber(find) ? transformFindUnitArray(defaultFindResults, find) : isArray(find) ? sortFindUnitArray(find, 1000) : find.find ? sortFindUnitArray(find.find, find.base || 1000) : transformFindUnitArray(defaultFindResults, find.base || 1000) : table ? sortFindUnitArray(table.map(function (_ref) {
    var pre = _ref.pre,
        power = _ref.power;
    return {
      pre: pre,
      exp: power
    };
  }), 10) : transformFindUnitArray(defaultFindResults, 1000);
  return function (value) {
    if (!value) {
      return unity;
    }

    var val = Math.abs(value);
    var lastIndex = results.length - 1;

    for (var i = 0; i < lastIndex; i++) {
      var obj = results[i];

      if (val >= obj.div) {
        return obj;
      }
    }

    return results[lastIndex] || unity;
  };
}

var isNaN = Number.isNaN,
    isFinite = Number.isFinite;

function createRounder(options) {
  var decOp = options.dec,
      fixed = options.fixed;
  var dec = decOp != null ? +decOp : 2;

  if (isNaN(dec) || !isFinite(dec) || dec < 0) {
    throw new TypeError('invalid "dec" option.');
  }

  if (fixed) {
    return function (num) {
      return num.toFixed(dec);
    };
  }

  var mul = pow(10, dec);
  return function (num) {
    return Math.round(num * mul) / mul;
  };
}

function defaultFormat(value, pre, unit) {
  var wholeUnit = "" + pre + unit;
  return "" + value + (wholeUnit ? " " + wholeUnit : '');
}

function createFormatter(options) {
  var _ref = options || {},
      unit = _ref.unit,
      find = _ref.find,
      round = _ref.round,
      output = _ref.output,
      deprecatedDec = _ref.dec,
      deprecatedFixed = _ref.fixed,
      deprecatedTable = _ref.table;

  var getUnit = isFunction(unit) ? unit : function () {
    return unit || '';
  };
  var findUnit = createUnitFinder(find, deprecatedTable);
  var roundNum = isFunction(round) ? round : createRounder(isNumber(round) ? {
    dec: round
  } : round || {
    dec: deprecatedDec,
    fixed: deprecatedFixed
  });
  var fmt = isFunction(output) ? output : defaultFormat;
  return function (value) {
    var unitObj = findUnit(value);
    var pre = unitObj.pre;
    var value2 = value / unitObj.div;
    var rounded = roundNum(value2);
    var result = fmt(rounded, pre, getUnit(value2, rounded, pre));
    return "" + result;
  };
}
function format(value, options) {
  return createFormatter(options)(value);
}

var reg = /^\s*(-?\d*\.?\d+(?:e[+-]?\d+)?)\s*([a-zA-Z]*)\s*$/;
function capture(input) {
  var result = reg.exec(input);

  if (!result) {
    return null;
  }

  var valueAsStr = result[1],
      unit = result[2];
  return [valueAsStr, unit];
}

var hasOwn = Object.prototype.hasOwnProperty;

var defaultFindItems = [{
  pre: 'meg',
  exp: 2
}, {
  pre: 'f',
  exp: -5
}, {
  pre: 'p',
  exp: -4
}, {
  pre: 'n',
  exp: -3
}, {
  pre: 'u',
  exp: -2
}, {
  pre: 'm',
  exp: -1
}, {
  pre: 'k',
  exp: 1
}, {
  pre: 'K',
  exp: 1
}, {
  pre: 'M',
  exp: 2
}, {
  pre: 'G',
  exp: 3
}, {
  pre: 'T',
  exp: 4
}];

function transformItems(find, base, unit) {
  var populate = unit ? function (result, pre, value) {
    result[pre] = value;
    result[pre + unit] = value;
  } : function (result, pre, value) {
    result[pre] = value;
  };
  return find.reduce(function (result, _ref) {
    var pre = _ref.pre,
        exp = _ref.exp;
    populate(result, pre, pow(base, exp));
    return result;
  }, {});
}

function createMulFinder(unit, find, table) {
  if (isFunction(find)) {
    return function (capturedUnit) {
      var result = find(capturedUnit);

      if (isNumber(result)) {
        if (result === 0) {
          throw new TypeError('Multiplier can\'t be zero');
        }

        return isNaN(result) ? null : {
          mul: result
        };
      }

      if (result == null) {
        return null;
      }

      if (typeof result !== 'object') {
        throw new TypeError(result + " if not a valid multiplier");
      }

      return result;
    };
  }

  var findTable = find ? isNumber(find) ? transformItems(defaultFindItems, find, unit) : isArray(find) ? transformItems(find, 1000, unit) : find.find ? transformItems(find.find, find.base || 1000, unit) : transformItems(defaultFindItems, find.base || 1000, unit) : table ? transformItems(table.map(function (_ref2) {
    var pre = _ref2.pre,
        power = _ref2.power;
    return {
      pre: pre,
      exp: power
    };
  }), 10, unit) : transformItems(defaultFindItems, 1000, unit);
  return function (capturedUnit) {
    if (capturedUnit === unit) {
      return {
        mul: 1
      };
    }

    if (hasOwn.call(findTable, capturedUnit)) {
      return {
        mul: findTable[capturedUnit]
      };
    }

    return null;
  };
}

function createParser(options) {
  var _ref = options || {},
      unit = _ref.unit,
      find = _ref.find,
      deprecatedTable = _ref.table;

  var findMul = createMulFinder(unit, find, deprecatedTable);
  return function (input) {
    if (isNumber(input)) {
      return isFinite(input) ? input : NaN;
    }

    if (!input) {
      return NaN;
    }

    var asString = "" + input;

    if (!asString) {
      return NaN;
    }

    var asNum = +asString;

    if (!isNaN(asNum)) {
      return isFinite(asNum) ? asNum : NaN;
    }

    var result = capture(asString);

    if (!result) {
      return NaN;
    }

    var valueAsStr = result[0],
        unit = result[1];
    var valueAsNum = +valueAsStr;

    if (!valueAsNum) {
      return valueAsNum === 0 ? 0 : NaN;
    }

    var mul = findMul(unit);

    if (!mul) {
      return NaN;
    }

    var mul2 = mul.mul;
    return mul2 ? valueAsNum * mul2 : NaN;
  };
}
function parse(input, options) {
  return createParser(options)(input);
}

export { MICRO, createFormatter, createParser, format, parse };
//# sourceMappingURL=gen-unit.mjs.map
